from django.shortcuts import render, redirect
from django.http import JsonResponse
from django.conf import settings
import time
import os
import rust_crud_api  # This is the module we built

db_url = settings.DATABASE_URL

def init_db_view(request):
    db_url = os.environ.get("DATABASE_URL")
    try:
        rust_crud_api.init_db(db_url)
        return JsonResponse({"message": "Database initialized"})
    except Exception as e:
        return JsonResponse({"error": str(e)}, status=500)

def register_view(request):
    """
    Handles user registration. Expects POST with 'name', 'email', and 'password'.
    Calls the Rust extension to create the user and then redirects to the login page.
    """
    db_url = settings.DATABASE_URL  # Defined in settings.py
    context = {}
    
    if request.method == 'POST':
        name = request.POST.get('name', '').strip()
        email = request.POST.get('email', '').strip()
        password = request.POST.get('password', '').strip()
        username = request.POST.get('username', '').strip()
        if not name or not email or not password:
            context['error'] = "Name, email, username and password are required."
            return render(request, 'myapp/register.html', context)
        
        try:
            # Create the user with the hashed password via the Rust extension.
            rust_crud_api.create_user(db_url, name, email, password, username)
            # Redirect to the login page after successful registration.
            return redirect('login')
        except Exception as e:
            context['error'] = f"An error occurred: {str(e)}"
            return render(request, 'myapp/register.html', context)
    
    return render(request, 'myapp/register.html', context)


def login_view(request):
    """
    Handles user login by verifying credentials using the Rust extension.
    If valid, stores user details in the session and redirects to the account page.
    """
    db_url = settings.DATABASE_URL
    context = {}
    
    if request.method == 'POST':
        email = request.POST.get('email', '').strip()
        password = request.POST.get('password', '').strip()
        
        if not email or not password:
            context['error'] = "Email and password are required."
            return render(request, 'myapp/login.html', context)
        
        try:
            # Verify the user's credentials using the Rust extension.
            is_valid = rust_crud_api.verify_user(db_url, email, password)
            if not is_valid:
                context['error'] = "Invalid email or password."
                return render(request, 'myapp/login.html', context)
            
            # Retrieve the user record to store additional info in session.
            # (For simplicity, we retrieve all users and select the matching one.)
            users = rust_crud_api.get_all_users(db_url)
            user = next((u for u in users if u.email.lower() == email.lower()), None)
            if not user:
                context['error'] = "User not found."
                return render(request, 'myapp/login.html', context)
            
            # Store user information in the session.
            request.session['user_id'] = user.id
            request.session['user_name'] = user.name
            request.session['user_email'] = user.email
            
            return redirect('account')
        except Exception as e:
            context['error'] = f"An error occurred: {str(e)}"
            return render(request, 'myapp/login.html', context)
    
    return render(request, 'myapp/login.html', context)

def account_view(request):
    """
    Display the account page for the logged-in user.
    """
    user_id = request.session.get('user_id')
    if not user_id:
        return redirect('login')
    db_url = settings.DATABASE_URL
    try:
        user = rust_crud_api.get_user(db_url, int(user_id))
        if user is None:
            return JsonResponse({'error': 'User not found.'}, status=404)
        return render(request, 'myapp/account.html', {'user': user})
    except Exception as e:
        return JsonResponse({'error': str(e)}, status=500)


def delete_account_view(request):
    """
    Delete the account of the logged-in user.
    """
    user_id = request.session.get('user_id')
    if not user_id:
        return redirect('login')
    db_url = settings.DATABASE_URL
    if request.method == 'POST':
        try:
            success = rust_crud_api.delete_user(db_url, int(user_id))
            if success:
                request.session.flush()  # Clear the session after deletion.
                return redirect('register')
            else:
                return JsonResponse({'error': 'Account deletion failed.'}, status=500)
        except Exception as e:
            return JsonResponse({'error': str(e)}, status=500)
    return render(request, 'myapp/delete_account.html')

def logout_view(request):
    # Clear all session data to log out the user.
    request.session.flush()
    # Optionally, render a confirmation page or redirect to login.
    return render(request, 'myapp/logout.html')

def group_list_view(request):
    """
    List all groups.
    """
    db_url = settings.DATABASE_URL
    context = {}
    try:
        groups = rust_crud_api.get_all_groups(db_url)
        context['groups'] = groups
    except Exception as e:
        context['error'] = f"An error occurred: {e}"
    return render(request, 'myapp/group_list.html', context)

def group_create_view(request):
    """
    Create a new group.
    """
    db_url = settings.DATABASE_URL
    context = {}
    if request.method == 'POST':
        group_name = request.POST.get('name', '').strip()
        if not group_name:
            context['error'] = "Group name is required."
            return render(request, 'myapp/group_create.html', context)
        try:
            rust_crud_api.create_group(db_url, group_name)
            return redirect('group_list')
        except Exception as e:
            context['error'] = f"An error occurred: {e}"
            return render(request, 'myapp/group_create.html', context)
    return render(request, 'myapp/group_create.html', context)

def group_update_view(request, group_id):
    """
    Update an existing group's name.
    """
    db_url = settings.DATABASE_URL
    context = {}
    try:
        group = rust_crud_api.get_group(db_url, int(group_id))
        if group is None:
            context['error'] = "Group not found."
            return render(request, 'myapp/group_update.html', context)
    except Exception as e:
        context['error'] = f"Error retrieving group: {e}"
        return render(request, 'myapp/group_update.html', context)
    
    if request.method == 'POST':
        new_name = request.POST.get('name', '').strip()
        if not new_name:
            context['error'] = "New group name is required."
            context['group'] = group
            return render(request, 'myapp/group_update.html', context)
        try:
            # This assumes you've implemented an `update_group` function in your Rust library.
            rust_crud_api.update_group(db_url, int(group_id), new_name)
            return redirect('group_list')
        except Exception as e:
            context['error'] = f"Error updating group: {e}"
            context['group'] = group
            return render(request, 'myapp/group_update.html', context)
    
    context['group'] = group
    return render(request, 'myapp/group_update.html', context)

def group_delete_view(request, group_id):
    """
    Delete a group.
    """
    db_url = settings.DATABASE_URL
    context = {}
    if request.method == 'POST':
        try:
            # This assumes you have a `delete_group` function in your Rust library.
            success = rust_crud_api.delete_group(db_url, int(group_id))
            if success:
                return redirect('group_list')
            else:
                context['error'] = "Deletion failed."
        except Exception as e:
            context['error'] = f"Error deleting group: {e}"
    else:
        try:
            group = rust_crud_api.get_group(db_url, int(group_id))
            context['group'] = group
        except Exception as e:
            context['error'] = f"Error retrieving group: {e}"
    return render(request, 'myapp/group_delete.html', context)

def group_detail_view(request, group_id):
    """
    Fetch a single group's details using the Rust extension and display them.
    """
    db_url = settings.DATABASE_URL
    context = {}
    try:
        group = rust_crud_api.get_group(db_url, group_id)
        if group is None:
            return HttpResponseNotFound("Group not found.")
        context['group'] = group
    except Exception as e:
        context['error'] = f"An error occurred: {e}"
    return render(request, 'myapp/group_detail.html', context)

def account_view(request):
    """
    Display the account page with user posts.
    """
    db_url = settings.DATABASE_URL
    user_id = request.session.get('user_id')
    if not user_id:
        return redirect('login')
    
    try:
        # Retrieve user info, e.g., using get_user (or session data)
        users = rust_crud_api.get_all_users(db_url)
        user = next((u for u in users if u.id == user_id), None)
        # Retrieve posts for this user (public posts)
        posts = rust_crud_api.get_user_posts(db_url, user_id)
    except Exception as e:
        return render(request, 'myapp/account.html', {'error': str(e)})
    
    return render(request, 'myapp/account.html', {'user': user, 'posts': posts})

def create_post_view(request):
    """
    Allow a logged-in user to create a new post.
    If a group is specified in the POST data, the post is associated with that group.
    """
    db_url = settings.DATABASE_URL
    user_id = request.session.get('user_id')
    if not user_id:
        return redirect('login')
    
    context = {}
    if request.method == 'POST':
        content = request.POST.get('content', '').strip()
        # Optionally, get a group_id if the post is for a group:
        group_id = request.POST.get('group_id')
        if group_id:
            try:
                group_id = int(group_id)
            except ValueError:
                group_id = None
        else:
            group_id = None
        
        if not content:
            context['error'] = "Post content cannot be empty."
            return render(request, 'myapp/create_post.html', context)
        
        try:
            rust_crud_api.create_post(db_url, user_id, group_id, content)
            return redirect('account')
        except Exception as e:
            context['error'] = f"Error creating post: {str(e)}"
    
    return render(request, 'myapp/create_post.html', context)

def feed_view(request):
    """
    Display a global feed of posts.
    """
    db_url = settings.DATABASE_URL
    try:
        posts = rust_crud_api.get_feed(db_url)
    except Exception as e:
        return render(request, 'myapp/feed.html', {'error': str(e)})
    
    return render(request, 'myapp/feed.html', {'posts': posts})
