use crate::models::Post;
use pyo3::prelude::*;
use pyo3::exceptions::PyRuntimeError;
use postgres::{Client, NoTls};
use chrono::{DateTime, Utc};
use std::time::SystemTime;

fn pg_err(e: postgres::Error) -> PyErr {
    PyRuntimeError::new_err(e.to_string())
}

/// Create a new post. If group_id is None, the post is for the user’s account or public feed.
/// Otherwise, it is posted to the specified group.
#[pyfunction(signature = (db_url, user_id, group_id=None, content=""))]
pub fn create_post(db_url: &str, user_id: i32, group_id: Option<i32>, content: &str) -> PyResult<()> {
    let mut client = Client::connect(db_url, NoTls).map_err(|e| PyRuntimeError::new_err(e.to_string()))?;
    client.execute(
        "INSERT INTO posts (user_id, group_id, content) VALUES ($1, $2, $3)",
        &[&user_id, &group_id.as_ref(), &content]
    ).map_err(|e| PyRuntimeError::new_err(e.to_string()))?;
    Ok(())
}

/// Retrieve posts for a given user. If group_id is None, returns posts that are not tied to a group,
/// which could be used for a public feed or the user’s own posts.
#[pyfunction]
pub fn get_user_posts(db_url: &str, user_id: i32) -> PyResult<Vec<String>> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;
    let rows = client.query(
        "SELECT content FROM posts WHERE user_id = $1 AND group_id IS NULL ORDER BY created_at DESC",
        &[&user_id]
    ).map_err(pg_err)?;
    
    let posts = rows.iter().map(|row| row.get(0)).collect();
    Ok(posts)
}

/// Retrieve posts for a given group.
#[pyfunction]
pub fn get_group_posts(db_url: &str, group_id: i32) -> PyResult<Vec<String>> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;
    let rows = client.query(
        "SELECT content FROM posts WHERE group_id = $1 ORDER BY created_at DESC",
        &[&group_id]
    ).map_err(pg_err)?;
    
    let posts = rows.iter().map(|row| row.get(0)).collect();
    Ok(posts)
}

/// Retrieve a global feed of posts (optionally filtering by user_id if needed).
#[pyfunction]
pub fn get_feed(db_url: &str) -> PyResult<Vec<Post>> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;
    let rows = client.query(
        "SELECT p.id, p.user_id, u.name, p.content, p.created_at
         FROM posts p
         JOIN users u ON p.user_id = u.id
         WHERE p.group_id IS NULL
         ORDER BY p.created_at DESC",
        &[]
    ).map_err(pg_err)?;

    let mut posts = Vec::new();
    for row in rows {
        let id: Option<i32> = row.get(0);
        let user_id: i32 = row.get(1);
        let user_name: Option<String> = Some(row.get(2));
        let content: String = row.get(3);
        let created_at: SystemTime = row.get(4);  // Retrieves timestamp from DB
        let created_at = DateTime::<Utc>::from(created_at).to_rfc3339();  // Convert to String
        posts.push(Post {
            id,
            user_id,
            user_name,
            content,
            created_at,  // Now a properly formatted String
        });
    }
    Ok(posts)
}
