use crate::models::User;
use pyo3::prelude::*;
use pyo3::exceptions::PyRuntimeError;
use postgres::{Client, NoTls};
use crate::auth::hash_password;
use crate::auth::verify_password;

fn pg_err(e: postgres::Error) -> PyErr {
    PyRuntimeError::new_err(e.to_string())
}


/// Create a new user by inserting into the database.
#[pyfunction]
pub fn create_user(db_url: &str, name: &str, email: &str, password: &str, username: &str) -> PyResult<()> {
    let password_hash = hash_password(password)?;
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;
    client.execute(
        "INSERT INTO users (name, email, password_hash, username) VALUES ($1, $2, $3, $4)",
        &[&name, &email, &password_hash, &username]
    ).map_err(pg_err)?;
    Ok(())
}

/// Retrieve a user by ID. Returns None if the user is not found.
#[pyfunction]
pub fn get_user(db_url: &str, user_id: i32) -> PyResult<Option<User>> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;
    let row_opt = client.query_opt(
        "SELECT id, name, email, username FROM users WHERE id = $1",
        &[&user_id]
    ).map_err(pg_err)?;
    
    if let Some(row) = row_opt {
        let user = User {
            id: row.get(0),
            name: row.get(1),
            email: row.get(2),
            username: row.get(3),
        };
        Ok(Some(user))
    } else {
        Ok(None)
    }
}

/// Retrieve all users from the database.
#[pyfunction]
pub fn get_all_users(db_url: &str) -> PyResult<Vec<User>> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;
    let rows = client.query("SELECT id, name, email, username FROM users", &[])
        .map_err(pg_err)?;
    
    let mut users = Vec::new();
    for row in rows {
        users.push(User {
            id: row.get(0),
            name: row.get(1),
            email: row.get(2),
            username: row.get(3),
        });
    }
    Ok(users)
}

/// Update an existing user with a new name and email.
#[pyfunction]
pub fn update_user(db_url: &str, user_id: i32, new_name: &str, new_email: &str) -> PyResult<()> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;
    let updated = client.execute(
        "UPDATE users SET name = $1, email = $2 WHERE id = $3",
        &[&new_name, &new_email, &user_id]
    ).map_err(pg_err)?;
    if updated == 0 {
        Err(PyRuntimeError::new_err("User not found"))
    } else {
        Ok(())
    }
}

/// Delete a user by ID. Returns true if a user was deleted.
#[pyfunction]
pub fn delete_user(db_url: &str, user_id: i32) -> PyResult<bool> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;
    let deleted = client.execute(
        "DELETE FROM users WHERE id = $1",
        &[&user_id]
    ).map_err(pg_err)?;
    Ok(deleted > 0)
}

///Verify credentials
///Retrieve a stored password hash for the email and compare.
#[pyfunction]
pub fn verify_user(db_url: &str, email: &str, password: & str) -> PyResult<bool> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;
    let row_opt = client.query_opt(
        "SELECT password_hash FROM users WHERE email = $1",
        &[&email]
        ).map_err(pg_err)?;
        if let Some(row) = row_opt {
            let stored_hash: String = row.get(0);
            verify_password(&stored_hash, password)
            } else {
                Ok(false)
        }
}
